@@ -22,8 +22,6 @@
 # @author: see AUTHORS file
 
 """Imports for the openpyxl package."""
-import warnings
-
 from openpyxl.xml import LXML
 
 from openpyxl.workbook import Workbook
@@ -31,11 +29,11 @@ from openpyxl.reader.excel import load_workbook
 
 
 # constants
-__version__ = '2.0.5'
+__version__ = '2.1.0'
 
 __author__ = 'Eric Gazoni'
 __license__ = 'MIT/Expat'
 __author_email__ = 'eric.gazoni@gmail.com'
 __maintainer_email__ = 'openpyxl-users@googlegroups.com'
 __url__ = 'http://openpyxl.readthedocs.org'
-__downloadUrl__ = "http://bitbucket.org/ericgazoni/openpyxl/downloads"
+__downloadUrl__ = "http://bitbucket.org/openpyxl/openpyxl/downloads"
