@@ -1,43 +1,32 @@
 from __future__ import absolute_import
 # Copyright (c) 2010-2014 openpyxl
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
-#
-# @license: http://www.opensource.org/licenses/mit-license.php
-# @author: see AUTHORS file
 
 """Worksheet is the 2nd-level container in Excel."""
 
 
 # Python stdlib imports
+from itertools import islice, chain
 import re
+from inspect import isgenerator
 
 # compatibility imports
-from openpyxl.compat import range
+from openpyxl.compat import (
+    unicode,
+    range,
+    basestring,
+    iteritems,
+    deprecated
+)
 
 # package imports
 import openpyxl.cell
 from openpyxl.cell import (
     coordinate_from_string,
     COORD_RE,
+    ABSOLUTE_RE,
     column_index_from_string,
-    get_column_letter
+    get_column_letter,
+    Cell
 )
 from openpyxl.exceptions import (
     SheetTitleException,
@@ -50,16 +39,14 @@ from openpyxl.units import (
     DEFAULT_COLUMN_WIDTH,
     DEFAULT_ROW_HEIGHT
 )
-from openpyxl.styles import Style, DEFAULTS as DEFAULTS_STYLE
+from openpyxl.styles import DEFAULTS as DEFAULTS_STYLE
 from openpyxl.formatting import ConditionalFormatting
-from openpyxl.namedrange import NamedRangeContainingValue
-from openpyxl.compat import OrderedDict, unicode, range, basestring
-from openpyxl.compat.itertools import iteritems
+from openpyxl.workbook.names.named_range import NamedRange
 
 from .header_footer import HeaderFooter
 from .relationship import Relationship
 from .page import PageSetup, PageMargins
-from .dimensions import ColumnDimension, RowDimension
+from .dimensions import ColumnDimension, RowDimension, DimensionHolder
 from .protection import SheetProtection
 from .filters import AutoFilter
 
@@ -76,6 +63,38 @@ class SheetView(object):
     pass
 
 
+def range_boundaries(range_string):
+    """
+    Convert a range string into a tuple of boundaries:
+    (min_col, min_row, max_col, max_row)
+    Cell coordinates will be converted into a range with the cell at both end
+    """
+    m = ABSOLUTE_RE.match(range_string)
+    min_col, min_row, sep, max_col, max_row = m.groups()
+    min_col = column_index_from_string(min_col)
+    min_row = int(min_row)
+
+    if max_col is None or max_row is None:
+        max_col = min_col
+        max_row = min_row
+    else:
+        max_col = column_index_from_string(max_col)
+        max_row = int(max_row)
+
+    return min_col, min_row, max_col, max_row
+
+
+def cells_from_range(range_string):
+    """
+    Get individual addresses for every cell in a range.
+    Yields one row at a time.
+    """
+    min_col, min_row, max_col, max_row = range_boundaries(range_string)
+    for row in range(min_row, max_row+1):
+        yield tuple('%s%d' % (get_column_letter(col), row)
+                    for col in range(min_col, max_col+1))
+
+
 class Worksheet(object):
     """Represents a worksheet.
 
@@ -119,7 +138,7 @@ class Worksheet(object):
         else:
             self.title = title
         self.row_dimensions = {}
-        self.column_dimensions = OrderedDict([])
+        self.column_dimensions = DimensionHolder([])
         self.page_breaks = []
         self._cells = {}
         self._styles = {}
@@ -141,8 +160,8 @@ class Worksheet(object):
         self.print_gridlines = False
         self.show_summary_below = True
         self.show_summary_right = True
-        self.default_row_dimension = RowDimension()
-        self.default_column_dimension = ColumnDimension()
+        self.default_row_dimension = RowDimension(worksheet=self)
+        self.default_column_dimension = ColumnDimension(worksheet=self)
         self._auto_filter = AutoFilter()
         self._freeze_panes = None
         self.paper_size = None
@@ -150,6 +169,8 @@ class Worksheet(object):
         self.orientation = None
         self.xml_source = None
         self.conditional_formatting = ConditionalFormatting()
+        self.vba_code = {}
+        self.vba_controls = None
 
     def __repr__(self):
         return self.repr_format % self.title
@@ -162,11 +183,15 @@ class Worksheet(object):
     def encoding(self):
         return self._parent.encoding
 
+    @deprecated('this method is private and should not be called directly')
     def garbage_collect(self):
+        self._garbage_collect()
+
+    def _garbage_collect(self):
         """Delete cells that are not storing a value."""
         delete_list = []
         for coordinate, cell in iteritems(self._cells):
-            if (not cell.merged and cell.value in ('', None)
+            if (cell.value in ('', None)
             and cell.comment is None
             and (coordinate not in self._styles or cell.style == DEFAULTS_STYLE)):
                 delete_list.append(coordinate)
@@ -189,13 +214,17 @@ class Worksheet(object):
         if self.bad_title_char_re.search(value):
             msg = 'Invalid character found in sheet title'
             raise SheetTitleException(msg)
-        value = self.unique_sheet_name(value)
+        value = self._unique_sheet_name(value)
         if len(value) > 31:
             msg = 'Maximum 31 characters allowed in sheet title'
             raise SheetTitleException(msg)
         self._title = value
 
+    @deprecated('this method is private and should not be called directly')
     def unique_sheet_name(self, value):
+        return self._unique_sheet_name(value)
+
+    def _unique_sheet_name(self, value):
         # check if sheet_name already exists
         # do this *before* length check
         sheets = self._parent.get_sheet_names()
@@ -294,15 +323,17 @@ class Worksheet(object):
             new_cell = openpyxl.cell.Cell(self, column, row)
             self._cells[coordinate] = new_cell
             if column not in self.column_dimensions:
-                self.column_dimensions[column] = ColumnDimension(column)
+                self.column_dimensions[column] = ColumnDimension(column, worksheet=self)
             if row not in self.row_dimensions:
-                self.row_dimensions[row] = RowDimension(row)
+                self.row_dimensions[row] = RowDimension(row, worksheet=self)
         return self._cells[coordinate]
 
     def __getitem__(self, key):
         """Convenience access by Excel style address"""
         if isinstance(key, slice):
-            return self.range("{0}:{1}".format(key.start, key.stop))
+            return self.iter_rows("{0}:{1}".format(key.start, key.stop))
+        if ":" in key:
+            return self.iter_rows(key)
         return self._get_cell(key)
 
     def __setitem__(self, key, value):
@@ -318,6 +349,13 @@ class Worksheet(object):
         else:
             return 0
 
+    @property
+    def min_row(self):
+        if self.row_dimensions:
+            return min(self.row_dimensions)
+        else:
+            return 1
+
     @property
     def max_row(self):
         return self.get_highest_row()
@@ -333,15 +371,120 @@ class Worksheet(object):
         else:
             return 1
 
+    @property
+    def min_col(self):
+        if self.column_dimensions:
+            return max([column_index_from_string(column_index)
+                        for column_index in self.column_dimensions])
+        else:
+            return 1
+
     @property
     def max_column(self):
         return self.get_highest_column()
 
     def calculate_dimension(self):
         """Return the minimum bounding range for all cells containing data."""
+        return '%s%d:%s%d' % (
+            get_column_letter(1),
+            self.min_row,
+            get_column_letter(self.max_column or 1),
+            self.max_row or 1)
 
-        return 'A1:%s%d' % (get_column_letter(self.max_column or 1), self.max_row or 1)
 
+    @property
+    def dimensions(self):
+        return self.calculate_dimension()
+
+
+    def iter_rows(self, range_string=None, row_offset=0, column_offset=0):
+        """
+        Returns a squared range based on the `range_string` parameter,
+        using generators.
+        If no range is passed, will iterate over all cells in the worksheet
+
+        :param range_string: range of cells (e.g. 'A1:C4')
+        :type range_string: string
+
+        :param row_offset: additional rows (e.g. 4)
+        :type row: int
+
+        :param column_offset: additonal columns (e.g. 3)
+        :type column: int
+
+        :rtype: generator
+        """
+        if range_string is not None:
+            min_col, min_row, max_col, max_row = range_boundaries(range_string.upper())
+        else:
+            min_col, min_row, max_col, max_row = (1, 1, self.max_column, self.max_row)
+        return self.get_squared_range(min_col + column_offset,
+                                      min_row + row_offset,
+                                      max_col + column_offset,
+                                      max_row + row_offset)
+
+
+    def get_squared_range(self, min_col, min_row, max_col, max_row):
+        """Returns a 2D array of cells
+
+        :param min_col: smallest column index (1-based index)
+        :type min_col: int
+
+        :param min_row: smallest row index (1-based index)
+        :type min_row: int
+
+        :param max_col: largest column index (1-based index)
+        :type max_col: int
+
+        :param max_row: smallest row index (1-based index)
+        :type max_row: int
+
+        :rtype: generator
+        """
+        # Column name cache is very important in large files.
+        cache = dict((col, get_column_letter(col)) for col in range(min_col, max_col+1))
+        rows = []
+        for row in range(min_row, max_row+1):
+            yield tuple(self._get_cell('%s%d' % (cache[col], row))
+                        for col in range(min_col, max_col+1))
+
+
+    def get_named_range(self, range_string):
+        """
+        Returns a 2D array of cells, with optional row and column offsets.
+
+        :param range_string: `named range` name
+        :type range_string: string
+
+        :rtype: tuples of tuples of :class:`openpyxl.cell.Cell
+        """
+        named_range = self._parent.get_named_range(range_string)
+        if named_range is None:
+            msg = '%s is not a valid range name' % range_string
+            raise NamedRangeException(msg)
+        if not isinstance(named_range, NamedRange):
+            msg = '%s refers to a value, not a range' % range_string
+            raise NamedRangeException(msg)
+
+        result = []
+        for destination in named_range.destinations:
+            worksheet, cells_range = destination
+
+            if worksheet is not self:
+                msg = 'Range %s is not defined on worksheet %s' % \
+                    (cells_range, self.title)
+                raise NamedRangeException(msg)
+
+            for row in self.iter_rows(cells_range):
+                result.extend(row)
+
+        return tuple(result)
+
+
+    @deprecated("""
+    Use .iter_rows() working with coordinates 'A1:D4',
+    and .get_squared_range() when working with indices (1, 1, 4, 4)
+    and .get_named_range() for named ranges""")
     def range(self, range_string, row=0, column=0):
         """Returns a 2D array of cells, with optional row and column offsets.
 
@@ -357,77 +500,38 @@ class Worksheet(object):
         :rtype: tuples of tuples of :class:`openpyxl.cell.Cell`
 
         """
-        if ':' in range_string:
-            # R1C1 range
-            result = []
-            min_range, max_range = range_string.split(':')
-            min_col, min_row = coordinate_from_string(min_range)
-            max_col, max_row = coordinate_from_string(max_range)
-            if column:
-                min_col = get_column_letter(
-                    column_index_from_string(min_col) + column)
-                max_col = get_column_letter(
-                    column_index_from_string(max_col) + column)
-            min_col = column_index_from_string(min_col)
-            max_col = column_index_from_string(max_col)
-            cache_cols = {}
-            for col in range(min_col, max_col + 1):
-                cache_cols[col] = get_column_letter(col)
-            rows = range(min_row + row, max_row + row + 1)
-            cols = range(min_col, max_col + 1)
-            for row in rows:
-                new_row = []
-                for col in cols:
-                    new_row.append(self.cell('%s%s' % (cache_cols[col], row)))
-                result.append(tuple(new_row))
-            return tuple(result)
+        _rs = range_string.upper()
+        m = ABSOLUTE_RE.match(_rs)
+         # R1C1 range
+        if m is not None:
+            rows = self.iter_rows(_rs, row_offset=row, column_offset=column)
+            return tuple(row for row in rows)
         else:
-            try:
-                return self.cell(coordinate=range_string, row=row,
-                                 column=column)
-            except CellCoordinatesException:
-                pass
+            return self.get_named_range(range_string)
 
-            # named range
-            named_range = self._parent.get_named_range(range_string)
-            if named_range is None:
-                msg = '%s is not a valid range name' % range_string
-                raise NamedRangeException(msg)
-            if isinstance(named_range, NamedRangeContainingValue):
-                msg = '%s refers to a value, not a range' % range_string
-                raise NamedRangeException(msg)
-
-            result = []
-            for destination in named_range.destinations:
-
-                worksheet, cells_range = destination
-
-                if worksheet is not self:
-                    msg = 'Range %s is not defined on worksheet %s' % \
-                        (cells_range, self.title)
-                    raise NamedRangeException(msg)
-
-                content = self.range(cells_range)
-
-                if isinstance(content, tuple):
-                    for cells in content:
-                        result.extend(cells)
-                else:
-                    result.append(content)
-
-            if len(result) == 1:
-                return result[0]
-            else:
-                return tuple(result)
 
+    @deprecated("Access styles directly from cells, columns or rows")
     def get_style(self, coordinate):
-        """Return the style object for the specified cell."""
-        if not coordinate in self._styles:
-            self._styles[coordinate] = self.parent.shared_styles.add(Style())
-        return self.parent.shared_styles[self._styles[coordinate]]
+        """Return a copy of the style object for the specified cell."""
+        try:
+            obj = self[coordinate]
+        except ValueError:
+            if isinstance(coordinate, int):
+                obj = self.row_dimensions[obj]
+            else:
+                obj = self.column_dimensions[obj]
+        return obj.style
 
+    @deprecated("Set styles directly on cells, columns or rows")
     def set_style(self, coordinate, style):
-        self._styles[coordinate] = self.parent.shared_styles.add(style)
+        try:
+            obj = self[coordinate]
+        except ValueError:
+            if isinstance(coordinate, int):
+                obj = self.row_dimensions[obj]
+            else:
+                obj = self.column_dimensions[obj]
+        obj.style = style
 
     def set_printer_settings(self, paper_size, orientation):
         """Set printer settings """
@@ -437,7 +541,11 @@ class Worksheet(object):
             raise ValueError("Values should be %s or %s" % (self.ORIENTATION_PORTRAIT, self.ORIENTATION_LANDSCAPE))
         self.page_setup.orientation = orientation
 
+    @deprecated('this method is private and should not be called directly')
     def create_relationship(self, rel_type):
+        return self._create_relationship(rel_type)
+
+    def _create_relationship(self, rel_type):
         """Add a relationship for this sheet."""
         rel = Relationship(rel_type)
         self.relationships.append(rel)
@@ -484,10 +592,10 @@ class Worksheet(object):
                     "'coordinate' or for 'start_row', 'start_column', 'end_row' *and* 'end_column'"
                 raise InsufficientCoordinatesException(msg)
             else:
-                range_string = '%s%s:%s%s' % (get_column_letter(start_column + 1),
-                                              start_row + 1,
-                                              get_column_letter(end_column + 1),
-                                              end_row + 1)
+                range_string = '%s%s:%s%s' % (get_column_letter(start_column),
+                                              start_row,
+                                              get_column_letter(end_column),
+                                              end_row)
         elif ":" not in range_string:
             if COORD_RE.match(range_string):
                 return # Single cell
@@ -496,22 +604,32 @@ class Worksheet(object):
         else:
             range_string = range_string.replace('$', '')
 
-        # Make sure top_left cell exists - is this necessary?
-        min_col, min_row = coordinate_from_string(range_string.split(':')[0])
-        max_col, max_row = coordinate_from_string(range_string.split(':')[1])
-        min_col = column_index_from_string(min_col)
-        max_col = column_index_from_string(max_col)
-        # Blank out the rest of the cells in the range
-        for col in range(min_col, max_col + 1):
-            for row in range(min_row, max_row + 1):
-                if not (row == min_row and col == min_col):
-                    # PHPExcel adds cell and specifically blanks it out if it doesn't exist
-                    self._get_cell('%s%s' % (get_column_letter(col), row)).value = None
-                    self._get_cell('%s%s' % (get_column_letter(col), row)).merged = True
-
         if range_string not in self._merged_cells:
             self._merged_cells.append(range_string)
 
+        cells = cells_from_range(range_string)
+        # only the top-left cell is preserved
+        for c in islice(chain.from_iterable(cells), 1, None):
+            if c in self._cells:
+                del self._cells[c]
+
+
+    @property
+    def merged_cells(self):
+        """Utility for checking whether a cell has been merged or not"""
+        cells = set()
+        for _range in self._merged_cells:
+            for row in cells_from_range(_range):
+                cells = cells.union(set(row))
+        return cells
+
+
+    @property
+    def merged_cell_ranges(self):
+        """Public attribute for which cells have been merged"""
+        return self._merged_cells
+
+
     def unmerge_cells(self, range_string=None, start_row=None, start_column=None, end_row=None, end_column=None):
         """ Remove merge on a cell range.  Range is a cell range (e.g. A1:E1) """
         if not range_string:
@@ -520,7 +638,7 @@ class Worksheet(object):
                     "'coordinate' or for 'start_row', 'start_column', 'end_row' *and* 'end_column'"
                 raise InsufficientCoordinatesException(msg)
             else:
-                range_string = '%s%s:%s%s' % (get_column_letter(start_column + 1), start_row + 1, get_column_letter(end_column + 1), end_row + 1)
+                range_string = '%s%s:%s%s' % (get_column_letter(start_column), start_row, get_column_letter(end_column), end_row)
         elif len(range_string.split(':')) != 2:
             msg = "Range must be a cell range (e.g. A1:E1)"
             raise InsufficientCoordinatesException(msg)
@@ -529,27 +647,19 @@ class Worksheet(object):
 
         if range_string in self._merged_cells:
             self._merged_cells.remove(range_string)
-            min_col, min_row = coordinate_from_string(range_string.split(':')[0])
-            max_col, max_row = coordinate_from_string(range_string.split(':')[1])
-            min_col = column_index_from_string(min_col)
-            max_col = column_index_from_string(max_col)
-            # Mark cell as unmerged
-            for col in range(min_col, max_col + 1):
-                for row in range(min_row, max_row + 1):
-                    if not (row == min_row and col == min_col):
-                        self._get_cell('%s%s' % (get_column_letter(col), row)).merged = False
+
         else:
             msg = 'Cell range %s not known as merged.' % range_string
             raise InsufficientCoordinatesException(msg)
 
-    def append(self, list_or_dict):
+    def append(self, iterable):
         """Appends a group of values at the bottom of the current sheet.
 
         * If it's a list: all values are added in order, starting from the first column
         * If it's a dict: values are assigned to the columns indicated by the keys (numbers or letters)
 
-        :param list_or_dict: list or dict containing values to append
-        :type list_or_dict: list/tuple or dict
+        :param iterable: list, range or generator, or dict containing values to append
+        :type iterable: list/tuple/range/generator or dict
 
         Usage:
 
@@ -557,36 +667,50 @@ class Worksheet(object):
         * **or** append({'A' : 'This is A1', 'C' : 'This is C1'})
         * **or** append({1 : 'This is A1', 3 : 'This is C1'})
 
-        :raise: TypeError when list_or_dict is neither a list/tuple nor a dict
+        :raise: TypeError when iterable is neither a list/tuple nor a dict
 
         """
         row_idx = self.max_row + 1
-        if isinstance(list_or_dict, (list, tuple)):
-            for col_idx, content in enumerate(list_or_dict, 1):
-                self.cell(row=row_idx, column=col_idx).value = content
 
-        elif isinstance(list_or_dict, dict):
-            for col_idx, content in iteritems(list_or_dict):
+        if (isinstance(iterable, (list, tuple, range))
+            or isgenerator(iterable)):
+            for col_idx, content in enumerate(iterable, 1):
+                col = get_column_letter(col_idx)
+                if col not in self.column_dimensions:
+                    self.column_dimensions[col] = ColumnDimension(col)
+                cell = Cell(self, col, row_idx, content)
+                self._cells['%s%d' % (col, row_idx)] = cell
+
+        elif isinstance(iterable, dict):
+            for col_idx, content in iteritems(iterable):
                 if isinstance(col_idx, basestring):
                     col_idx = column_index_from_string(col_idx)
                 self.cell(row=row_idx, column=col_idx).value = content
 
         else:
-            raise TypeError('list_or_dict must be a list or a dict')
+            self._invalid_row(iterable)
+        self.row_dimensions[row_idx] = RowDimension(row_idx)
+
+    def _invalid_row(self, iterable):
+        raise TypeError('Value must be a list, tuple, range or generator, or a dict. Supplied value is {0}'.format(
+            type(iterable))
+                        )
 
     @property
     def rows(self):
-        return self.range(self.calculate_dimension())
+        """Iterate over all rows in the worksheet"""
+        return tuple(row for row in self.iter_rows())
 
     @property
     def columns(self):
-        max_row = self.get_highest_row()
+        """Iterate over all columns in the worksheet"""
+        max_row = self.max_row
+        min_row = 1
         cols = []
-        for col_idx in range(self.get_highest_column()):
-            col = get_column_letter(col_idx + 1)
-            res = self.range('%s1:%s%d' % (col, col, max_row))
-            cols.append(tuple([x[0] for x in res]))
-
+        for col_idx in range(self.max_column):
+            cells = self.get_squared_range(col_idx + 1, min_row, col_idx + 1, max_row)
+            col = chain.from_iterable(cells)
+            cols.append(tuple(col))
         return tuple(cols)
 
     def point_pos(self, left=0, top=0):
